==> create-fly-1.21.8-6.0.8-2-sources.jar: com/zurrtum/create/client/mixin/BlockDustParticleMixin.java <==
package com.zurrtum.create.client.mixin;

import com.llamalad7.mixinextras.injector.wrapoperation.Operation;
import com.llamalad7.mixinextras.injector.wrapoperation.WrapOperation;
import com.llamalad7.mixinextras.sugar.Local;
import com.llamalad7.mixinextras.sugar.Share;
import com.llamalad7.mixinextras.sugar.ref.LocalRef;
import com.zurrtum.create.client.infrastructure.model.WrapperBlockStateModel;
import com.zurrtum.create.content.decoration.copycat.CopycatBlock;
import net.minecraft.class_1058;
import net.minecraft.class_1087;
import net.minecraft.class_2248;
import net.minecraft.class_2338;
import net.minecraft.class_2382;
import net.minecraft.class_2680;
import net.minecraft.class_638;
import net.minecraft.class_727;
import net.minecraft.class_773;
import org.spongepowered.asm.mixin.Mixin;
import org.spongepowered.asm.mixin.Unique;
import org.spongepowered.asm.mixin.injection.At;

@Mixin(class_727.class)
public abstract class BlockDustParticleMixin {
    @Unique
    private static final class_2382[] DIRECTIONS = new class_2382[]{new class_2382(0, 0, -1), new class_2382(0, 0, 1), new class_2382(-1, 0, 0), new class_2382(1, 0, 0), new class_2382(
        -1,
        0,
        -1
    ), new class_2382(1, 0, -1), new class_2382(1, 0, 1), new class_2382(-1, 0, 1), new class_2382(0, -1, 0), new class_2382(0, 1, 0), new class_2382(
        0,
        -1,
        -1
    ), new class_2382(0, -1, 1), new class_2382(-1, -1, 0), new class_2382(1, -1, 0), new class_2382(-1, -1, -1), new class_2382(1, -1, -1), new class_2382(1, -1, 1), new class_2382(
        -1,
        -1,
        1
    )};

    @Unique
    private static class_2338 findPos(class_638 world, class_2338 pos, class_2680 state) {
        class_2680 target = world.method_8320(pos);
        if (target == state) {
            return pos;
        }
        class_2338.class_2339 position = pos.method_25503();
        for (class_2382 move : DIRECTIONS) {
            target = world.method_8320(position.method_35831(pos, move));
            if (target == state) {
                return position;
            }
        }
        return pos;
    }

    @WrapOperation(method = "<init>(Lnet/minecraft/client/world/ClientWorld;DDDDDDLnet/minecraft/block/BlockState;Lnet/minecraft/util/math/BlockPos;)V", at = @At(value = "INVOKE", target = "Lnet/minecraft/client/render/block/BlockModels;getModelParticleSprite(Lnet/minecraft/block/BlockState;)Lnet/minecraft/client/texture/Sprite;"))
    private class_1058 onParticle(
        class_773 models,
        class_2680 state,
        Operation<class_1058> original,
        @Local(argsOnly = true) class_638 world,
        @Local(argsOnly = true) class_2338 pos,
        @Share("pos") LocalRef<class_2338> blockPos
    ) {
        class_1087 model = models.method_3335(state);
        if (WrapperBlockStateModel.unwrapCompat(model) instanceof WrapperBlockStateModel wrapper) {
            blockPos.set(findPos(world, pos, state));
            return wrapper.particleSpriteWithInfo(world, blockPos.get(), state);
        } else {
            return model.method_68511();
        }
    }

    @WrapOperation(method = "<init>(Lnet/minecraft/client/world/ClientWorld;DDDDDDLnet/minecraft/block/BlockState;Lnet/minecraft/util/math/BlockPos;)V", at = @At(value = "INVOKE", target = "Lnet/minecraft/block/BlockState;isOf(Lnet/minecraft/block/Block;)Z"))
    private boolean isGrass(
        class_2680 state,
        class_2248 block,
        Operation<Boolean> original,
        @Local(argsOnly = true) class_638 world,
        @Share("pos") LocalRef<class_2338> blockPos
    ) {
        if (state.method_26204() instanceof CopycatBlock) {
            class_2338 pos = blockPos.get();
            if (pos != null) {
                state = CopycatBlock.getMaterial(world, pos);
            }
        }
        return original.call(state, block);
    }
}

==> create-fly-6.0.8-1.21.10-2-sources.jar: com/zurrtum/create/client/mixin/BlockDustParticleMixin.java <==
package com.zurrtum.create.client.mixin;

import com.llamalad7.mixinextras.injector.wrapoperation.Operation;
import com.llamalad7.mixinextras.injector.wrapoperation.WrapOperation;
import com.llamalad7.mixinextras.sugar.Local;
import com.llamalad7.mixinextras.sugar.Share;
import com.llamalad7.mixinextras.sugar.ref.LocalRef;
import com.zurrtum.create.client.infrastructure.model.WrapperBlockStateModel;
import com.zurrtum.create.content.decoration.copycat.CopycatBlock;
import net.minecraft.class_1058;
import net.minecraft.class_1087;
import net.minecraft.class_2248;
import net.minecraft.class_2338;
import net.minecraft.class_2382;
import net.minecraft.class_2680;
import net.minecraft.class_638;
import net.minecraft.class_727;
import net.minecraft.class_773;
import org.spongepowered.asm.mixin.Mixin;
import org.spongepowered.asm.mixin.Unique;
import org.spongepowered.asm.mixin.injection.At;

@Mixin(class_727.class)
public abstract class BlockDustParticleMixin {
    @Unique
    private static final class_2382[] DIRECTIONS = new class_2382[]{new class_2382(0, 0, -1), new class_2382(0, 0, 1), new class_2382(-1, 0, 0), new class_2382(1, 0, 0), new class_2382(
        -1,
        0,
        -1
    ), new class_2382(1, 0, -1), new class_2382(1, 0, 1), new class_2382(-1, 0, 1), new class_2382(0, -1, 0), new class_2382(0, 1, 0), new class_2382(0, -1, -1), new class_2382(
        0,
        -1,
        1
    ), new class_2382(-1, -1, 0), new class_2382(1, -1, 0), new class_2382(-1, -1, -1), new class_2382(1, -1, -1), new class_2382(
        1,
        -1,
        1
    ), new class_2382(-1, -1, 1)};

    @Unique
    private static class_2338 findPos(class_638 world, class_2338 pos, class_2680 state) {
        class_2680 target = world.method_8320(pos);
        if (target == state) {
            return pos;
        }
        class_2338.class_2339 position = pos.method_25503();
        for (class_2382 move : DIRECTIONS) {
            target = world.method_8320(position.method_35831(pos, move));
            if (target == state) {
                return position;
            }
        }
        return pos;
    }

    @WrapOperation(method = "<init>(Lnet/minecraft/client/world/ClientWorld;DDDDDDLnet/minecraft/block/BlockState;Lnet/minecraft/util/math/BlockPos;)V", at = @At(value = "INVOKE", target = "Lnet/minecraft/client/render/block/BlockModels;getModelParticleSprite(Lnet/minecraft/block/BlockState;)Lnet/minecraft/client/texture/Sprite;"))
    private static class_1058 onParticle(
        class_773 models,
        class_2680 state,
        Operation<class_1058> original,
        @Local(argsOnly = true) class_638 world,
        @Local(argsOnly = true) class_2338 pos,
        @Share("pos") LocalRef<class_2338> blockPos
    ) {
        class_1087 model = models.method_3335(state);
        if (WrapperBlockStateModel.unwrapCompat(model) instanceof WrapperBlockStateModel wrapper) {
            blockPos.set(findPos(world, pos, state));
            return wrapper.particleSpriteWithInfo(world, blockPos.get(), state);
        } else {
            return model.method_68511();
        }
    }

    @WrapOperation(method = "<init>(Lnet/minecraft/client/world/ClientWorld;DDDDDDLnet/minecraft/block/BlockState;Lnet/minecraft/util/math/BlockPos;)V", at = @At(value = "INVOKE", target = "Lnet/minecraft/block/BlockState;isOf(Lnet/minecraft/block/Block;)Z"))
    private boolean isGrass(
        class_2680 state,
        class_2248 block,
        Operation<Boolean> original,
        @Local(argsOnly = true) class_638 world,
        @Share("pos") LocalRef<class_2338> blockPos
    ) {
        if (state.method_26204() instanceof CopycatBlock) {
            class_2338 pos = blockPos.get();
            if (pos != null) {
                state = CopycatBlock.getMaterial(world, pos);
            }
        }
        return original.call(state, block);
    }
}
